#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 26 13:49:28 2019

@author: isegura
"""

from phase1 import Student
from phase1 import StudentsList

import queue #it is Python module to implement queues

class TreeNode:
  
  def __init__(self,elem=None):
    self.elem=elem
    self.leftChild=None
    self.rightChild=None
    self.parent=None
    
    
class StudentsTree():
    
  def __init__(self):
    self.root=None
    
  def size(self):
    """Returns the number of nodes"""
    return self._size(self.root)
      
  def _size(self,currentNode):
    if currentNode==None:
      return 0
    
    return 1 + self._size(currentNode.leftChild) + self._size(currentNode.rightChild)
   
    
  def height(self):
    """Returns the height of the tree"""
    return self._height(self.root)
  
  def _height(self,currentNode):
    if currentNode==None:
      return -1
    
    return 1 + max(self._height(currentNode.leftChild), self._height(currentNode.rightChild))

  def depth(self,currentNode):
    """Returns the depth of a node"""
  
    if currentNode==None:
      return 0
    
    return 1 + self.depth(currentNode.parent)

  
  def preorder(self):
    print('pre-order traversal')  
    self._preorder(self.root)
    print()
    
  def _preorder(self,currentNode):
    if currentNode!=None:
      print(str(currentNode.elem))
      self._preorder(currentNode.leftChild)
      self._preorder(currentNode.rightChild)
      
      
  def postorder(self):
    print('post-order traversal')  
    self._postorder(self.root)
    print()
    
  def _postorder(self,currentNode):
    if currentNode!=None:
      self._postorder(currentNode.leftChild)
      self._postorder(currentNode.rightChild)
      print(str(currentNode.elem))
      
      
      
  def inorder(self):
    print('in-order traversal')  
    self._inorder(self.root)
    print()
    
  def _inorder(self,currentNode):
    if currentNode!=None:
      self._inorder(currentNode.leftChild)
      print(str(currentNode.elem))
      self._inorder(currentNode.rightChild)
      
      
  def getOrderedList(self):
    result=StudentsList()
    self._getOrderedList(result,self.root)
    return result
    
  def _getOrderedList(self,result,currentNode):
    if currentNode!=None:
      self._getOrderedList(result,currentNode.leftChild)
      result.addLast(currentNode.elem)
      self._getOrderedList(result,currentNode.rightChild)
      
  def draw(self):
    """Fucntion to draw a tree"""
    self._draw('',self.root,False)
    print()
      
  def _draw(self,prefix, node, isLeft):
    if node !=None:
      self._draw(prefix + "     ", node.rightChild, False)
      print(prefix + ("|-- ") + str(node.elem.email))
      self._draw(prefix + "     ", node.leftChild, True)
    
  def levelorder(self):
    if self.root==None:
      print('tree is empty')
      return
    print('level-order traversal')
    q=queue.Queue()
    q.put(self.root) #we save the root
    
    while q.empty()==False:
      current=q.get() #dequeue
      print(str(current.elem), end=' ')
      if current.leftChild:
        q.put(current.leftChild)
      if current.rightChild:
        q.put(current.rightChild)
    print()
  
  def insert(self,x):
    """inserts a new node, with element x, into the tree"""
    if self.root==None:
      self.root=TreeNode(x)
    else:
      self.insertNode(self.root,x)
            
  def insertNode(self,node,x):
    """Inserts a new node (with the element x) inside of the subtree node"""
    if node.elem.email==x.email:
        # Duplicate elements are not allowed
        print(x.email,'already exists!!!')
        return
            
    if x.email<node.elem.email:
        if node.leftChild!=None:
            self.insertNode(node.leftChild,x)
        else:
            newNode=TreeNode(x)
            node.leftChild=newNode
            newNode.parent=node
    else: #x>node.elem
        if node.rightChild!=None:
            self.insertNode(node.rightChild,x)
        else:
            newNode=TreeNode(x)
            node.rightChild=newNode
            newNode.parent=node

  def search(self,x):
    return self.searchNode(self.root,x)
    
  def searchNode(self,node,x):
    """Auxiliary method to search a node with value x"""
    if node is None:
      return False
    
    if node.elem.email==x.email:
        return True
    
    if x.email<node.elem.email:
        return self.searchNode(node.leftChild,x)
    
    if x.email>node.elem.email:
        return self.searchNode(node.rightChild,x)
 
  def find(self,x):
    """Returns the ndoe whose element is x. If it is not found, it returns None"""
    return self.findNode(self.root,x)

  def findNode(self,node,x):
    if node is None:
        return None
    
    if node.elem.email==x.email:
        return node
    
    if x.email<node.elem.email:
        return self.findNode(node.leftChild,x)
    
    if x.email>node.elem.email:
        return self.findNode(node.rightChild,x)      
      
  def remove(self,x):
    """Searches and removes the node whose element is x"""
    node=self.find(x)
    if node is None:
        print(x.email,' does not exist!!!')
        return
    print('removing ', x.email)
    self.removeNode(node)
    
  
  def removeNode(self,node):    
    """Auxiliary method to remove the node which takes as parameter"""
    #First case: no children
    if node.leftChild is None and node.rightChild is None:
        parent_node=node.parent
        if parent_node is not None:
            if parent_node.leftChild==node:
                parent_node.leftChild=None
            else:
                parent_node.rightChild=None
            node.parent=None
        else:
            self.root=None
        return
    
    #Second case: only one child
    if node.leftChild is not None and node.rightChild is None:
        
        parent_node=node.parent
        if parent_node is not None:
            if parent_node.leftChild==node:
                parent_node.leftChild=node.leftChild
            else:
                parent_node.rightChild=node.leftChild
            node.leftChild.parent=parent_node
        else:
            self.root=node.leftChild
        return
    
     #Second case: only one child
    if node.leftChild is None and node.rightChild is not None:
        
        parent_node=node.parent
        if parent_node is not None:
            if parent_node.leftChild==node:
                parent_node.leftChild=node.rightChild
            else:
                parent_node.rightChild=node.rightChild
            node.rightChild.parent=parent_node
        else:
            self.root=node.rightChild
        return
        
    #Third case: two children
    successor=node.rightChild
    while successor.leftChild is not None:
        successor=successor.leftChild
        
    #we replace the node's elem by the successor's elem
    node.elem=successor.elem
    #we remove the succesor from the tree
    self.removeNode(successor)
#        
  def copySocialNetwork(self,listOfStudent):
      for i in range(listOfStudent.size):
          student=listOfStudent.getAt(i)
          self.insert(student)
  
  def deleteByNumberOfBlocks(self,num):
      result=self.getOrderedList()
      for i in range(result.size):
          student=result.getAt(i)
          if student.blocks>=num:
              self.remove(student)
              
              

#####    
        





