# -*- coding: utf-8 -*-


class Vertex:
    def __init__(self, email):
      """This constructor function takes the id of the vertes and creates
      and empty list to store its adjacent vertices"""
      self.email = email
      self.friends = []
        
    def addFriend(self, vertex):
      """This functions takes the id of a vertex 
      adds its id as a new adjacent vertex for the vertex"""
      if vertex.email not in self.friends:
          #we only have to save the id of the vertex
          self.friends.append(vertex.email)
          self.friends = sorted(self.friends)
     
        
    def getFriends(self):
      """Returns the list of adjacent vertices"""
      return self.friends
      
    def __str__(self):
      return str(self.email) + ' connectedTo: ' + str([x for x in self.friends])




class StudentGraph:
    def __init__(self,list_of_emails):
        self.students = {}
        for e in list_of_emails:
            self.addStudent(e)

    def addStudent(self,email):
        newStudent = Vertex(email)
        self.students[email] = newStudent

    def getDirectFriends(self,email):
        if email in self.students:
            vertex=self.students[email]
            return vertex.friends
        else:
            return None
    
    def getStudents(self):
        return self.students.keys()


    def __contains__(self,email):
        return email in self.students
 
    def addFriends(self,start,end):
        if start not in self.students:
            self.addStudent(start)
        if end not in self.students:
            self.addStudent(end)
        self.students[start].addFriend(self.students[end])
        self.students[end].addFriend(self.students[start])
        

    def suggestedFriends(self,email):
        friends=self.getDirectFriends(email)
        
        visited={}
        for x in self.getStudents():
            visited[x]=False
        
        suggested=[]
        for x in friends:
            self.depth(x,suggested,visited)
        
        suggested.remove(email)
        suggested=set(suggested)-set(friends)
        
        return suggested    
            
    def depth(self,email,suggested,visited):
        if email not in suggested and visited[email]==False:
            suggested.append(email)
            visited[email]=True
            for x in self.getDirectFriends(email):
                self.depth(x,suggested,visited)
                
            
        
        
    
    def __iter__(self):
        return iter(self.students.values())
      

    def __str__(self):
        result=''
        for x in iter(self):
            result+=str(x)+'\n'
            
        return result
    
    

  